==> ota_tools_r32.7.1_aarch64.tbz2: Linux_for_Tegra/tools/ota_tools/version_upgrade/ota_install_r32i_images_to_sd.sh <==
#!/bin/bash

# Copyright (c) 2019-2020, NVIDIA CORPORATION.  All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

# This is a script to make necessary partitions on SD card/USB disk and
# write correspoding images from OTA package into them.

#      Partition             Size
# -------------------
# | cpu-bootloader  |        524288
# -------------------
# | bootloader-dtb  |        524288
# -------------------
# |   secure-os     |       3145728
# -------------------
# |      eks        |       2097152
# -------------------
# |    adsp-fw      |       4194304
# -------------------
# |     bpmp-fw     |        618144
# -------------------
# |  bpmp-fw-dtb    |        512000
# -------------------
# |    sce-fw       |       2097152
# -------------------
# |   kernel-dtb    |        524288
# -------------------
# |    recovery     |      66060288
# -------------------
# |  recovery-dtb   |        524288
# -------------------
# | kernel-bootctrl |        262144
# -------------------

_PARTITIONS=(cpu-bootloader bootloader-dtb secure-os eks adsp-fw bpmp-fw bpmp-fw-dtb sce-fw kernel-dtb recovery recovery-dtb kernel-bootctrl)
_PARTITIONS_DEV_NODE=("" "" "" "" "" "" "" "" "" "" "" "")
_PARTITIONS_IMAGE_NAME=("" "" "" "" "" "" "" "" "" "" "" "")
_PARTITIONS_IMAGE_SIZE=(0 0 0 0 0 0 0 0 0 0 0 0)
_PARTITIONS_IMAGE_SHA1SUM=("" "" "" "" "" "" "" "" "" "" "" "")
_PARTITIONS_SIZE=(0 0 0 0 0 0 0 0 0 0 0 0)
_PARTITIONS_SIZE_TOTAL=0
_DISK_TYPE=
_WORK_DIR=
_K_BYTES=1024

parse_index_file()
{
	local index_file=$1
	local index_array=

	readarray index_array < ${index_file}

	local lines_num=${#index_array[@]}
	ota_log "Number of lines is $lines_num"

	local max_lines_index=$((lines_num - 1))
	ota_log "max_lines_index=${max_lines_index}"

	local partitions_num=${#_PARTITIONS[@]}
	ota_log "partitions_num=${partitions_num}"

	local max_partitions_index=$((partitions_num - 1))
	ota_log "max_partitions_index=${max_partitions_index}"

	for i in `seq 0 ${max_partitions_index}`
	do
		local partition=${_PARTITIONS[${i}]}
		local found=0
		local item=

		for j in `seq 0 ${max_lines_index}`
		do
			item=${index_array[${j}]}
			local part_name=`echo $item | cut -d, -f 2 | sed 's/^ //g' - | cut -d: -f 3`
			if [ "${part_name}" = "${partition}" ];then
				found=1
				break
			fi
		done

		if [ ${found} -eq 1 ];then
			local part_size=`echo $item | cut -d, -f 4 | sed 's/^ //g' -`
			local image_name=`echo $item | cut -d, -f 5 | sed 's/^ //g' -`
			local image_size=`echo $item | cut -d, -f 6 | sed 's/^ //g' -`
			local image_sha1sum=`echo $item | cut -d, -f 8 | sed 's/^ //g' -`
			if [ ${part_size} -eq 0 ];then
				ota_log "Warning: the size of partition ${partition} is zero !!!"
				return 1
			else
				_PARTITIONS_SIZE[${i}]=${part_size}
				_PARTITIONS_SIZE_TOTAL=$(($_PARTITIONS_SIZE_TOTAL + ${part_size}))
			fi
			if [ "${image_name}" = "" ];then
				ota_log "Warning: the image name for partition ${partition} is null"
				return 1
			else
				_PARTITIONS_IMAGE_NAME[${i}]="${image_name}"
			fi
			if [ "${image_size}" = "" ];then
				ota_log "Warning: the size of ${image_name} for partition ${partition} is null"
				return 1
			else
				_PARTITIONS_IMAGE_SIZE[${i}]=${image_size}
			fi
			if [ "${image_sha1sum}" = "" ];then
				ota_log "Warning: the sha1 chksum of ${image_name} for partition ${partition} is null"
				return 1
			else
				_PARTITIONS_IMAGE_SHA1SUM[${i}]=${image_sha1sum}
			fi

		else
			ota_log "Partition ${partition} is not found in index file"
		fi
	done
	return 0
}

create_partitions_on_storage_disk()
{
	local disk=$1
	local start_number=`ls -al /dev/* | grep -c ${disk}`

	sudo parted "${disk}" unit "B" print free | grep "Free Space" | tail -1 | sed 's/^[ \t]*//g' - >/tmp/free.txt
	local free_start=`cat /tmp/free.txt | sed 's/B  / /g' - | cut -d\  -f 1`
	local free_end=`cat /tmp/free.txt | sed 's/B  / /g' - | cut -d\  -f 2`
	local free_size=`cat /tmp/free.txt | sed 's/B  / /g' - | cut -d\  -f 3`

	ota_log "free space: start=${free_start} end=${free_end} size=${free_size}"

	if [ ${_PARTITIONS_SIZE_TOTAL} -gt ${free_size} ];then
		ota_log "Warning: free space is not enough to create these partitions"
		return 1
	fi

	local partitions_num=${#_PARTITIONS[@]}
	ota_log "partitions_num=${partitions_num}"

	local max_partitions_index=$((partitions_num - 1))
	ota_log "max_partitions_index=${max_partitions_index}"

	local start_offset=${free_start}
	for i in `seq 0 ${max_partitions_index}`
	do
		local partition=${_PARTITIONS[${i}]}
		local partition_size=${_PARTITIONS_SIZE[${i}]}
		local end_offset=$((${start_offset} + ${partition_size}))

		ota_log "sudo parted ${disk} unit "B" mkpart primary ${start_offset} ${end_offset}"
		sudo parted -s ${disk} unit "B" mkpart primary ${start_offset} ${end_offset}

		ota_log "sudo parted ${disk} name ${start_number} ${partition}"
		sudo parted -s ${disk} name ${start_number} "${partition}"

		if [ "${_DISK_TYPE}" = "sdcard" ];then
			_PARTITIONS_DEV_NODE[${i}]="${disk}p${start_number}"
		else
			_PARTITIONS_DEV_NODE[${i}]="${disk}${start_number}"
		fi

		start_offset=`sudo parted "${disk}" unit "B" print free | grep "Free Space" | tail -1 | sed 's/[ ][ ]*/ /g' | cut -d\  -f 2 | sed 's/B$//g'`
		start_number=$((${start_number} + 1))
	done
	return 0
}

read_part_opt()
{
	local infile=$1
	local outfile=$2
	local size=$3

	if [ ! -e "${infile}" ];then
		ota_log "Input file ${infile} is not found"
		return 1
	fi

	if [ ${size} -eq 0 ];then
		ota_log "The size of bytes to be read is ${size}"
		return 1
	fi

	local block=$((${size} / ${_K_BYTES}))
	local remainder=$((${size} % ${_K_BYTES}))
	local offset=$((${block} * ${_K_BYTES}))

	ota_log "Reading ${size} bytes from ${infile} to : 1KB block=${block} remainder=${remainder} offset=${offset}"

	if [ ${block} -gt 0 ];then
		dd if=${infile} of=${outfile} bs=1K count=${block}
		sync
	fi
	if [ ${remainder} -gt 0 ];then
		dd if=${infile} of=${outfile} bs=1 skip=${offset} seek=${offset} count=${remainder}
		sync
	fi
	return 0
}

write_images_into_storage_partitions()
{
	local disk=${1}
	local partitions_num=${#_PARTITIONS[@]}
	local max_partitions_index=$((partitions_num - 1))

	for i in `seq 0 ${max_partitions_index}`
	do
		local part_name=${_PARTITIONS[${i}]}
		local image_name=${_PARTITIONS_IMAGE_NAME[${i}]}
		local image_path=

		if [ "${part_name}" = "kernel-bootctrl" ];then
			image_path=${_WORK_DIR}/${R32_3_IMAGE_DIR}/${image_name}.update
		else
			image_path=${_WORK_DIR}/${R32i_IMAGE_DIR}/${image_name}
		fi

		# verify sha1 chksum
		local image_sha1sum=${_PARTITIONS_IMAGE_SHA1SUM[${i}]}
		local image_sha1sum_file=${_WORK_DIR}/${image_name}.sha1sum

		if [ "${part_name}" != "kernel-bootctrl" ];then
			echo -n "${image_sha1sum}" >${image_sha1sum_file}
			sha1sum_verify "${image_path}" "${image_sha1sum_file}"
		fi

		# write image
		ota_log "Writing ${image_path} into partition ${part_name}"
		local part_node=${_PARTITIONS_DEV_NODE[${i}]}
		ota_log "dd if=${image_path} of=${part_node} conv=notrunc"
		sudo dd if=${image_path} of=${part_node} conv=notrunc
		sync

		# read back and verify it
		ota_log "Read back from partition ${part_name} and verify it"
		local image_tmp=${_WORK_DIR}/${image_name}.tmp
		local image_size=${_PARTITIONS_IMAGE_SIZE[${i}]}
		# sudo dd if=${part_node} of=${image_tmp} bs=1 count=${image_size}
		read_part_opt "${part_node}" "${image_tmp}" "${image_size}"
		if [ $? -ne 0 ];then
			ota_log "Failed to read ${image_size} bytes from ${part_node} to ${image_tmp}"
			return 1
		fi
		sync
		if [ "${part_name}" = "kernel-bootctrl" ];then
			sha1sum "${image_path}" | cut -d\  -f 1 >${image_sha1sum_file}
		fi

		sha1sum_verify "${image_tmp}" "${image_sha1sum_file}"
		ota_log "Writing partition ${part_name} done"
	done
	return 0
}

ota_install_r32i_images_to_sd()
{
	local work_dir=$1
	if [ ! -d "${work_dir}" ];then
		ota_log "OTA work directory ${work_dir} is not found"
		return 1
	fi
	_WORK_DIR=${work_dir}

	local storage_disk=$2
	if [ ! -e "${storage_disk}" ];then
		ota_log "Storage disk ${storage_disk} is not found"
		return 1
	fi
	local sdcard=`echo ${storage_disk} | grep mmcblk`
	if [ "${sdcard}" = "" ];then
		_DISK_TYPE="usbdisk"
	else
		_DISK_TYPE="sdcard"
	fi

	# parse the flash index file
	local index_file="${work_dir}/${INDEX_FILE_PATH}"
	if [ ! -f "${index_file}" ];then
		ota_log "${index_file} is not found"
		return 1
	fi

	parse_index_file "${index_file}"
	if [ ${_PARTITIONS_SIZE_TOTAL} -eq 0 ];then
		ota_log "Warning: total size of partitions is zero"
		return 1
	fi

	create_partitions_on_storage_disk ${storage_disk}
	if [ $? -ne 0 ];then
		ota_log "Failed to call \"create_partitions_on_storage_disk ${storage_disk}\""
		return 1
	fi

	write_images_into_storage_partitions ${storage_disk}
	if [ $? -ne 0 ];then
		ota_log "Failed to call \"write_images_into_storage_partitions ${storage_disk}\""
		return 1
	fi

	return 0
}

==> ota_tools_R35.1.0_aarch64.tbz2: Linux_for_Tegra/tools/ota_tools/version_upgrade/ota_install_r32i_images_to_sd.sh <==
#!/bin/bash

# Copyright (c) 2019-2021, NVIDIA CORPORATION.  All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

# This is a script to make necessary partitions on SD card/USB disk and
# write correspoding images from OTA package into them.

#      Partition             Size
# -------------------
# | cpu-bootloader  |        524288
# -------------------
# | bootloader-dtb  |        524288
# -------------------
# |   secure-os     |       3145728
# -------------------
# |      eks        |       2097152
# -------------------
# |    adsp-fw      |       4194304
# -------------------
# |     bpmp-fw     |        618144
# -------------------
# |  bpmp-fw-dtb    |        512000
# -------------------
# |    sce-fw       |       2097152
# -------------------
# |   kernel-dtb    |        524288
# -------------------
# |    recovery     |      66060288
# -------------------
# |  recovery-dtb   |        524288
# -------------------
# | kernel-bootctrl |        262144
# -------------------

_PARTITIONS=(cpu-bootloader bootloader-dtb secure-os eks adsp-fw bpmp-fw bpmp-fw-dtb sce-fw kernel-dtb recovery recovery-dtb kernel-bootctrl)
_PARTITIONS_DEV_NODE=("" "" "" "" "" "" "" "" "" "" "" "")
_PARTITIONS_IMAGE_NAME=("" "" "" "" "" "" "" "" "" "" "" "")
_PARTITIONS_IMAGE_SIZE=(0 0 0 0 0 0 0 0 0 0 0 0)
_PARTITIONS_IMAGE_SHA1SUM=("" "" "" "" "" "" "" "" "" "" "" "")
_PARTITIONS_SIZE=(0 0 0 0 0 0 0 0 0 0 0 0)
_PARTITIONS_SIZE_TOTAL=0
_DISK_TYPE=
_WORK_DIR=
_K_BYTES=1024

parse_index_file()
{
	local index_file=$1
	local index_array=

	readarray index_array < "${index_file}"

	local lines_num=${#index_array[@]}
	ota_log "Number of lines is $lines_num"

	local max_lines_index=$((lines_num - 1))
	ota_log "max_lines_index=${max_lines_index}"

	local partitions_num=${#_PARTITIONS[@]}
	ota_log "partitions_num=${partitions_num}"

	local max_partitions_index=$((partitions_num - 1))
	ota_log "max_partitions_index=${max_partitions_index}"

	for i in $(seq 0 ${max_partitions_index})
	do
		local partition=${_PARTITIONS[${i}]}
		local found=0
		local item=

		for j in $(seq 0 ${max_lines_index})
		do
			item=${index_array[${j}]}
			local part_name=$(echo "$item" | cut -d, -f 2 | sed 's/^ //g' - | cut -d: -f 3)
			if [ "${part_name}" = "${partition}" ];then
				found=1
				break
			fi
		done

		if [ ${found} -eq 1 ];then
			local part_size=$(echo "$item" | cut -d, -f 4 | sed 's/^ //g' -)
			local image_name=$(echo "$item" | cut -d, -f 5 | sed 's/^ //g' -)
			local image_size=$(echo "$item" | cut -d, -f 6 | sed 's/^ //g' -)
			local image_sha1sum=$(echo "$item" | cut -d, -f 8 | sed 's/^ //g' -)
			if [ "${part_size}" -eq 0 ];then
				ota_log "Warning: the size of partition ${partition} is zero !!!"
				return 1
			else
				_PARTITIONS_SIZE[${i}]=${part_size}
				_PARTITIONS_SIZE_TOTAL=$(($_PARTITIONS_SIZE_TOTAL + ${part_size}))
			fi
			if [ "${image_name}" = "" ];then
				ota_log "Warning: the image name for partition ${partition} is null"
				return 1
			else
				_PARTITIONS_IMAGE_NAME[${i}]="${image_name}"
			fi
			if [ "${image_size}" = "" ];then
				ota_log "Warning: the size of ${image_name} for partition ${partition} is null"
				return 1
			else
				_PARTITIONS_IMAGE_SIZE[${i}]=${image_size}
			fi
			if [ "${image_sha1sum}" = "" ];then
				ota_log "Warning: the sha1 chksum of ${image_name} for partition ${partition} is null"
				return 1
			else
				_PARTITIONS_IMAGE_SHA1SUM[${i}]=${image_sha1sum}
			fi

		else
			ota_log "Partition ${partition} is not found in index file"
		fi
	done
	return 0
}

create_partitions_on_storage_disk()
{
	local disk=$1
	local start_number=$(ls -al /dev/* | grep -c "${disk}")

	sudo parted "${disk}" unit "B" print free | grep "Free Space" | tail -1 | sed 's/^[ \t]*//g' - >/tmp/free.txt
	local free_start=$(cat /tmp/free.txt | sed 's/B  / /g' - | cut -d\  -f 1)
	local free_end=$(cat /tmp/free.txt | sed 's/B  / /g' - | cut -d\  -f 2)
	local free_size=$(cat /tmp/free.txt | sed 's/B  / /g' - | cut -d\  -f 3)

	ota_log "free space: start=${free_start} end=${free_end} size=${free_size}"

	if [ ${_PARTITIONS_SIZE_TOTAL} -gt "${free_size}" ];then
		ota_log "Warning: free space is not enough to create these partitions"
		return 1
	fi

	local partitions_num=${#_PARTITIONS[@]}
	ota_log "partitions_num=${partitions_num}"

	local max_partitions_index=$((partitions_num - 1))
	ota_log "max_partitions_index=${max_partitions_index}"

	local start_offset=${free_start}
	for i in $(seq 0 ${max_partitions_index})
	do
		local partition=${_PARTITIONS[${i}]}
		local partition_size=${_PARTITIONS_SIZE[${i}]}
		local end_offset=$((${start_offset} + ${partition_size}))

		ota_log "sudo parted ${disk} unit "B" mkpart primary ${start_offset} ${end_offset}"
		sudo parted -s "${disk}" unit "B" mkpart primary "${start_offset}" ${end_offset}

		ota_log "sudo parted ${disk} name ${start_number} ${partition}"
		sudo parted -s "${disk}" name "${start_number}" "${partition}"

		if [ "${_DISK_TYPE}" = "sdcard" ];then
			_PARTITIONS_DEV_NODE[${i}]="${disk}p${start_number}"
		else
			_PARTITIONS_DEV_NODE[${i}]="${disk}${start_number}"
		fi

		start_offset=$(sudo parted "${disk}" unit "B" print free | grep "Free Space" | tail -1 | sed 's/[ ][ ]*/ /g' | cut -d\  -f 2 | sed 's/B$//g')
		start_number=$((${start_number} + 1))
	done
	return 0
}

read_part_opt()
{
	local infile=$1
	local outfile=$2
	local size=$3

	if [ ! -e "${infile}" ];then
		ota_log "Input file ${infile} is not found"
		return 1
	fi

	if [ "${size}" -eq 0 ];then
		ota_log "The size of bytes to be read is ${size}"
		return 1
	fi

	local block=$((${size} / ${_K_BYTES}))
	local remainder=$((${size} % ${_K_BYTES}))
	local offset=$((${block} * ${_K_BYTES}))

	ota_log "Reading ${size} bytes from ${infile} to : 1KB block=${block} remainder=${remainder} offset=${offset}"

	if [ ${block} -gt 0 ];then
		dd if=${infile} of=${outfile} bs=1K count=${block}
		sync
	fi
	if [ ${remainder} -gt 0 ];then
		dd if=${infile} of=${outfile} bs=1 skip=${offset} seek=${offset} count=${remainder}
		sync
	fi
	return 0
}

write_images_into_storage_partitions()
{
	local disk=${1}
	local partitions_num=${#_PARTITIONS[@]}
	local max_partitions_index=$((partitions_num - 1))

	for i in $(seq 0 ${max_partitions_index})
	do
		local part_name=${_PARTITIONS[${i}]}
		local image_name=${_PARTITIONS_IMAGE_NAME[${i}]}
		local image_path=

		if [ "${part_name}" = "kernel-bootctrl" ];then
			image_path=${_WORK_DIR}/${R32_3_IMAGE_DIR}/${image_name}.update
		else
			image_path=${_WORK_DIR}/${R32i_IMAGE_DIR}/${image_name}
		fi

		# verify sha1 chksum
		local image_sha1sum=${_PARTITIONS_IMAGE_SHA1SUM[${i}]}
		local image_sha1sum_file=${_WORK_DIR}/${image_name}.sha1sum

		if [ "${part_name}" != "kernel-bootctrl" ];then
			echo -n "${image_sha1sum}" >${image_sha1sum_file}
			sha1sum_verify "${image_path}" "${image_sha1sum_file}"
		fi

		# write image
		ota_log "Writing ${image_path} into partition ${part_name}"
		local part_node=${_PARTITIONS_DEV_NODE[${i}]}
		ota_log "dd if=${image_path} of=${part_node} conv=notrunc"
		sudo dd if=${image_path} of=${part_node} conv=notrunc
		sync

		# read back and verify it
		ota_log "Read back from partition ${part_name} and verify it"
		local image_tmp=${_WORK_DIR}/${image_name}.tmp
		local image_size=${_PARTITIONS_IMAGE_SIZE[${i}]}
		# sudo dd if=${part_node} of=${image_tmp} bs=1 count=${image_size}
		read_part_opt "${part_node}" "${image_tmp}" "${image_size}"
		if [ $? -ne 0 ];then
			ota_log "Failed to read ${image_size} bytes from ${part_node} to ${image_tmp}"
			return 1
		fi
		sync
		if [ "${part_name}" = "kernel-bootctrl" ];then
			sha1sum "${image_path}" | cut -d\  -f 1 >${image_sha1sum_file}
		fi

		sha1sum_verify "${image_tmp}" "${image_sha1sum_file}"
		ota_log "Writing partition ${part_name} done"
	done
	return 0
}

ota_install_r32i_images_to_sd()
{
	local work_dir=$1
	if [ ! -d "${work_dir}" ];then
		ota_log "OTA work directory ${work_dir} is not found"
		return 1
	fi
	_WORK_DIR=${work_dir}

	local storage_disk=$2
	if [ ! -e "${storage_disk}" ];then
		ota_log "Storage disk ${storage_disk} is not found"
		return 1
	fi
	local sdcard=$(echo "${storage_disk}" | grep mmcblk)
	if [ "${sdcard}" = "" ];then
		_DISK_TYPE="usbdisk"
	else
		_DISK_TYPE="sdcard"
	fi

	# parse the flash index file
	local index_file="${work_dir}/${INDEX_FILE_PATH}"
	if [ ! -f "${index_file}" ];then
		ota_log "${index_file} is not found"
		return 1
	fi

	parse_index_file "${index_file}"
	if [ ${_PARTITIONS_SIZE_TOTAL} -eq 0 ];then
		ota_log "Warning: total size of partitions is zero"
		return 1
	fi

	create_partitions_on_storage_disk "${storage_disk}"
	if [ $? -ne 0 ];then
		ota_log "Failed to call \"create_partitions_on_storage_disk ${storage_disk}\""
		return 1
	fi

	write_images_into_storage_partitions "${storage_disk}"
	if [ $? -ne 0 ];then
		ota_log "Failed to call \"write_images_into_storage_partitions ${storage_disk}\""
		return 1
	fi

	return 0
}
